import { HazelReader, HazelWriter } from "@skeldjs/util";
import { ExtractEventTypes } from "@skeldjs/events";
import { BaseRpcMessage, GameSettings } from "@skeldjs/protocol";
import { SpawnType, Color, ChatNoteType, StringNames } from "@skeldjs/constant";
import { PlayerCheckColorEvent, PlayerCheckNameEvent, PlayerCompleteTaskEvent, PlayerUseMovingPlatformEvent, PlayerMurderEvent, PlayerRemoveProtectionEvent, PlayerReportDeadBodyEvent, PlayerSendChatEvent, PlayerSendQuickChatEvent, PlayerSetColorEvent, PlayerSetHatEvent, PlayerSetNameEvent, PlayerSetPetEvent, PlayerSetSkinEvent, PlayerSetStartCounterEvent, PlayerStartMeetingEvent, PlayerSyncSettingsEvent, PlayerDieEvent, PlayerSetVisorEvent, PlayerSetNameplateEvent, PlayerCheckMurderEvent, PlayerSetRoleEvent, PlayerProtectEvent, PlayerCheckProtectEvent, PlayerShapeshiftEvent, PlayerRevertShapeshiftEvent } from "../events";
import { Networkable, NetworkableEvents, NetworkableConstructor } from "../Networkable";
import { Hostable } from "../Hostable";
import { PlayerData } from "../PlayerData";
import { BaseRole } from "../roles";
export interface PlayerControlData {
    isNew: boolean;
    playerId: number;
}
export declare type PlayerControlEvents<RoomType extends Hostable = Hostable> = NetworkableEvents<RoomType> & ExtractEventTypes<[
    PlayerCheckColorEvent<RoomType>,
    PlayerCheckMurderEvent<RoomType>,
    PlayerCheckNameEvent<RoomType>,
    PlayerCheckProtectEvent<RoomType>,
    PlayerCompleteTaskEvent<RoomType>,
    PlayerDieEvent<RoomType>,
    PlayerUseMovingPlatformEvent<RoomType>,
    PlayerMurderEvent<RoomType>,
    PlayerProtectEvent<RoomType>,
    PlayerRemoveProtectionEvent<RoomType>,
    PlayerReportDeadBodyEvent<RoomType>,
    PlayerRevertShapeshiftEvent<RoomType>,
    PlayerSendChatEvent<RoomType>,
    PlayerSendQuickChatEvent<RoomType>,
    PlayerSetColorEvent<RoomType>,
    PlayerSetHatEvent<RoomType>,
    PlayerSetNameEvent<RoomType>,
    PlayerSetNameplateEvent<RoomType>,
    PlayerSetPetEvent<RoomType>,
    PlayerSetRoleEvent<RoomType>,
    PlayerSetSkinEvent<RoomType>,
    PlayerSetStartCounterEvent<RoomType>,
    PlayerSetVisorEvent<RoomType>,
    PlayerShapeshiftEvent<RoomType>,
    PlayerStartMeetingEvent<RoomType>,
    PlayerSyncSettingsEvent<RoomType>
]>;
/**
 * Represents a player object for interacting with the game and other players.
 *
 * See {@link PlayerControlEvents} for events to listen to.
 */
export declare class PlayerControl<RoomType extends Hostable = Hostable> extends Networkable<PlayerControlData, PlayerControlEvents<RoomType>, RoomType> implements PlayerControlData {
    private lastStartCounter;
    /**
     * Whether the player was just spawned, or was spawned before joining.
     */
    isNew: boolean;
    /**
     * The player ID of the player.
     */
    playerId: number;
    /**
     * The player that this component belongs to.
     */
    player: PlayerData<RoomType>;
    /**
     * Whether or not this player has been protected bya guardian angel.
     */
    protectedByGuardian: boolean;
    /**
     * The player (i.e. a guardian angel) who has protected this player, if the
     * player is being protected..
     */
    guardianProtector?: PlayerData<RoomType>;
    /**
     * The player that this player has shapeshifted as, if any.
     */
    shapeshiftTarget?: PlayerData<RoomType>;
    private _protectedByGuardianTime;
    constructor(room: RoomType, spawnType: SpawnType, netId: number, ownerid: number, flags: number, data?: HazelReader | PlayerControlData);
    Awake(): void;
    getComponent<T extends Networkable>(component: NetworkableConstructor<T>): T | undefined;
    Deserialize(reader: HazelReader, spawn?: boolean): void;
    Serialize(writer: HazelWriter, spawn?: boolean): boolean;
    FixedUpdate(delta: number): void;
    HandleRpc(rpc: BaseRpcMessage): Promise<void>;
    private _handleCompleteTask;
    private _completeTask;
    private _rpcCompleteTask;
    /**
     * Mark one of this player's tasks as completed.
     * @param taskIdx The index of the task to complete. Note: this is not the
     * task ID but instead the index of the player's task, as in its position in
     * the {@link PlayerInfo.taskStates} array.
     *
     * Emits a {@link PlayerCompleteTaskEvent | `player.completetask`} event.
     *
     * @example
     * ```ts
     * client.me.control.completeTask(0);
     * ```
     */
    completeTask(taskIdx: number): void;
    private _handleSyncSettings;
    private _syncSettings;
    private _rpcSyncSettings;
    /**
     * Change the room's settings. Use {@link Hostable.setSettings} to pass a
     * partial game objects object. This is a host operation on official servers.
     *
     * Emits a {@link PlayerSyncSettingsEvent | `player.syncsettings`} event.
     *
     * @property settings The full game settings object to update the settings to.
     */
    syncSettings(settings: GameSettings): Promise<void>;
    private _handleCheckName;
    private _rpcCheckName;
    /**
     * Request for the host to check a name for this player and append numbers if
     * it's taken.
     * @param name The name to request.
     */
    checkName(name: string): Promise<void>;
    private _handleSetName;
    private _rpcSetName;
    /**
     * Update this player's name. This is a host operation on official servers.
     * Use {@link PlayerControl.checkName} if you are calling this as not the host.
     *
     * Emits a {@link PlayerSetNameEvent | `player.setname`} event.
     *
     * @param name The name to set this player's name to.
     */
    setName(name: string): Promise<void>;
    private _handleCheckColor;
    private _rpcCheckColor;
    /**
     * Request for the host to check a color for this player and change it if it's
     * taken.
     *
     * @param color The color to request.
     */
    checkColor(color: Color): Promise<void>;
    private _handleSetColor;
    private _rpcSetColor;
    /**
     * Update this player's color. This is a host operation on official servers.
     * Use {@link PlayerControl.checkColor} if you are calling this as not the host.
     *
     * Emits a {@link PlayerCheckNameEvent | `player.checkname`} event.
     *
     * @param color The color to set this player's name to.
     */
    setColor(color: Color): Promise<void>;
    private _handleReportDeadBody;
    private _rpcReportDeadBody;
    kill(reason: string): Promise<void>;
    private _handleMurderPlayer;
    private _rpcMurderPlayer;
    /**
     * Murder another a player. This operation can only be called if the player
     * is the impostor on official servers.
     *
     * Due to technical limitations, this operation cannot be canceled or reverted
     * without advanced "breaking game", therefore it is out of scope of a single
     * `.revert()` function on the event emitted.
     *
     * Emits a {@link PlayerMurderEvent | `player.murder`} event.
     *
     * @param victim The player to murder.
     * @returns
     */
    murderPlayer(victim: PlayerData): Promise<void>;
    private _checkMurderEndGame;
    private _handleSendChat;
    private _rpcSendChat;
    /**
     * Send a chat message as this player.
     *
     * Due to technical impossibilities, this event cannot be canceled or reverted.
     *
     * Emits a {@link PlayerSendChatEvent | `player.chat`} event.
     */
    sendChat(message: string): void;
    private _rpcSendChatNote;
    /**
     * Send a player chat note for this player, i.e. "player x voted, x remaining.".
     */
    sendChatNote(player: PlayerData, type: ChatNoteType): void;
    private _handleStartMeeting;
    private _startMeeting;
    private _rpcStartMeeting;
    /**
     * If you're the host, this will immediately begin a meeting
     * Start a meeting and begin the meeting
     *
     * Emits a {@link PlayerStartMeetingEvent | `player.startmeeting`} event.
     *
     * @param body The body that was reported, or "emergency" if it is an emergency meeting.
     * @param caller The player that called this meeting.
     */
    startMeeting(body: PlayerData | "emergency", caller?: PlayerData): Promise<void>;
    private _handleSetStartCounter;
    private _setStartCounter;
    private _rpcSetStartCounter;
    /**
     * Change the counter at the bottom of the screen while in the lobby,
     * usually counting down from 5 to 1. This is a host-only operation
     * on official servers.
     *
     * Emits a {@link PlayerSetStartCounterEvent | `player.setstartcounter`} event.
     *
     * @param counter The counter to set to.
     */
    setStartCounter(counter: number): Promise<void>;
    private _handleUsePlatform;
    private _usePlatform;
    /**
     * Use the moving platfrom on the map, i.e. the one on Airship.
     *
     * Emits a {@link MovingPlatformPlayerUpdateEvent | `movingplatform.playerupdate`} event.
     */
    usePlatform(): Promise<void>;
    private _handleSendQuickChat;
    private _rpcSendQuickChat;
    /**
     * Send a quick chat message as this player.
     *
     * Due to technical impossibilities, this event cannot be canceled or reverted.
     *
     * Emits a {@link PlayerSendChatEvent | `player.quickchat`} event.
     */
    sendQuickChat(message: PlayerData | StringNames, format?: (PlayerData | StringNames)[]): void;
    private _handleSetHat;
    private _rpcSetHat;
    /**
     * Update this player's hat. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetHatEvent | `player.sethat`} event.
     *
     * @param hatId The hat to set this player's hat to, see {@link Hat}.
     */
    setHat(hatId: string): Promise<void>;
    private _handleSetSkin;
    private _rpcSetSkin;
    /**
     * Update this player's skin. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetSkinEvent | `player.setskin`} event.
     *
     * @param skinId The skin to set this player's skin to, see {@link Skin}.
     */
    setSkin(skinId: string): Promise<void>;
    private _handleSetPet;
    private _rpcSetPet;
    /**
     * Update this player's pet. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetPetEvent | `player.setpet`} event.
     *
     * @param petId The pet to set this player's pet to, see {@link Pet}.
     */
    setPet(petId: string): Promise<void>;
    private _handleSetVisor;
    private _rpcSetVisor;
    /**
     * Update this player's visor. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetVisorEvent | `player.setvisor`} event.
     *
     * @param visorId The visor to set this player's visor to, see {@link Visor}.
     */
    setVisor(visorId: string): Promise<void>;
    private _handleSetNameplate;
    private _rpcSetNameplate;
    /**
     * Update this player's nameplate. This is not a host operation unless the
     * client does not own this player.
     *
     * Emits a {@link PlayerSetNameplateEvent | `player.setnameplate`} event.
     *
     * @param nameplateId The nameplate to set this player's nameplate to, see {@link Nameplate}.
     */
    setNameplate(nameplateId: string): Promise<void>;
    private _handleSetRole;
    private _rpcSetRole;
    setRole(roleCtr: typeof BaseRole): Promise<void>;
    private _handleProtectPlayer;
    private _addProtection;
    private _rpcProtectPlayer;
    protectPlayer(target: PlayerData, angelColor?: Color): Promise<void>;
    private _handleShapeshift;
    private _shapeshift;
    private _rpcShapeshift;
    shapeshift(target: PlayerData<RoomType>, doAnimation?: boolean): Promise<void>;
    revertShapeshift(): Promise<void>;
    private _handleCheckMurder;
    private _rpcCheckMurder;
    /**
     * Remove this player's protection from a guardian angel, if they have one.
     *
     * Emits a {@link PlayerRemoveProtectionEvent | `player.removeprotection`} event.
     * This is not networked, so other players will not see this update.
     *
     * @param timeout Whether or not this is because of a timeout.
     */
    removeProtection(timeout?: boolean): Promise<void>;
    private _protectIsValid;
    private _handleCheckProtect;
    private _rpcCheckProtect;
}
//# sourceMappingURL=PlayerControl.d.ts.map